VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsIniFile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'***************************************************************
'* clsIniFile - VB-based .Ini file manipulation class.         *
'* Written 01/05 Nerdley Noodleslicer (noodleslicer@yahoo.com) *
'***************************************************************
'* This is a class intended to be a Visual Basic replacement   *
'* for the ReadPrivateProfileString/WritePrivateProfileString  *
'* API calls for manipulating .Ini files.  In addition, you can*
'* delete sections and items in .ini files.  The .ini resides  *
'* in RAM as a virtual file in a user-defined structure that   *
'* you declare as Public in your main module.                  *
'* I've been able to test it on small and large .ini files     *
'* successfully.  I have timed this on different systems and   *
'* depending on the size of the .ini, your system speed and    *
'* other factors, it seems to me this class runs about 3 to 8  *
'* times slower than API commands. Since we're talking about   *
'* short times here, you shouldn't notice any major slowdown   *
'* using this class instead of the API calls in typical .ini   *
'* file access situations.  This class may not cover all .ini  *
'* situations, although I've tried to be thorough.  Please let *
'* me know how I can improve it, as I know it's almost cert-   *
'* ainly not perfect.  No one's code is, after all!  Thanks.   *
'***************************************************************

Option Explicit
Private ErrorMessage As String

Friend Function ReadFrom(Ini As IniFileType, ByVal Section As String, ItemKey As String, Optional Default As String = "") As String

'***************************************************************
'* returns a value string from specified section/item key.     *
'***************************************************************

   Dim SectionNdx As Long, ItemLineNdx As Long

   On Error GoTo ReadFromError

'  determine if the specified section exists.  if not, exit.
   Bracketize Section
   SectionNdx = SectionExists(Ini, Section)
   If SectionNdx = (-1) Then
      ReadFrom = Default
      Exit Function
   End If

'  determine if the item key exists.
   ItemLineNdx = ItemExists(Ini, SectionNdx, ItemKey)
   If ItemLineNdx = (-1) Then
      ReadFrom = Default
      Exit Function
   End If

'  since we've arrived here, it's an item key match.  get the value.
   ReadFrom = Right(Ini.IniFile(ItemLineNdx), Len(Ini.IniFile(ItemLineNdx)) - InStr(Ini.IniFile(ItemLineNdx), "="))

   Exit Function

ReadFromError:
   ErrorMessage = "Error " & CStr(Err.Number) & " (" & Err.Description & ") occurred while reading a value from " & _
                  "the data structure containing " & Ini.FileName & "." & vbCrLf & _
                  "Ini Section Name = " & Section & ", Item Key = " & ItemKey
   DisplayErrorMessage ErrorMessage, "ReadFrom"

End Function

Friend Sub WriteTo(Ini As IniFileType, ByVal Section As String, Item As String, Value As String)

'***************************************************************
'* writes a value string to specified section/item.            *
'***************************************************************

   Dim SectionNdx As Long, ItemLineNdx As Long, IniLine As String
   Dim ItemInsertionPoint As Long, i As Long

   On Error GoTo WriteToError
'  determine if section exists.
   Bracketize Section
   SectionNdx = SectionExists(Ini, Section)
   If SectionNdx = (-1) Then
'     if the section does not exist, append a new one to the end of the ini UDT.
      CreateNewSection Ini, Section
'     since a new section was just created, automatically create the itemkey also.
      Ini.TotalIniLines = Ini.TotalIniLines + 1
      ReDim Preserve Ini.IniFile(1 To Ini.TotalIniLines)
      Ini.IniFile(Ini.TotalIniLines) = Item & "=" & Trim(Value)
      Ini.NumItems(Ini.SectionCount) = 1
'     since we've created the new section and item entries, we can exit.
      Exit Sub
   Else
'     if the section exists, find out if the item key exists.
      ItemLineNdx = ItemExists(Ini, SectionNdx, Item)
'     if the item key exists, simply change the value and exit.
      If ItemLineNdx <> (-1) Then
         Ini.IniFile(ItemLineNdx) = Item & "=" & Trim(Value)
         Exit Sub
      Else
'        otherwise, insert a new item key/value entry just after the last non-blank line.
         ItemInsertionPoint = Ini.SectionIndex(SectionNdx) + Ini.NumItems(SectionNdx) + 1
         If Ini.IniFile(ItemInsertionPoint - 1) = "" Then
            ItemInsertionPoint = ItemInsertionPoint - 1
         End If
         ReDim Preserve Ini.IniFile(1 To Ini.TotalIniLines + 1)
'        move all lines from insertion point to end of file 'to the right' one slot.
         For i = Ini.TotalIniLines + 1 To ItemInsertionPoint + 1 Step -1
            Ini.IniFile(i) = Ini.IniFile(i - 1)
         Next i
'        place the new item key/value into the insertion point.
         Ini.IniFile(ItemInsertionPoint) = Item & "=" & Trim(Value)
         Ini.TotalIniLines = Ini.TotalIniLines + 1
         Ini.NumItems(SectionNdx) = Ini.NumItems(SectionNdx) + 1
'        adjust line# of any section headers following the section containing the just-added item.
         For i = SectionNdx + 1 To Ini.SectionCount
            Ini.SectionIndex(i) = Ini.SectionIndex(i) + 1
         Next i
      End If
   End If

   Exit Sub

WriteToError:
   ErrorMessage = "Error " & CStr(Err.Number) & " (" & Err.Description & ") occurred while writing a value to " & _
                  "the data structure containing " & Ini.FileName & "." & vbCrLf & _
                  "Ini Section Name = " & Section & ", Item Key = " & Item
   DisplayErrorMessage ErrorMessage, "WriteTo"

End Sub

Friend Sub LoadIniFile(Ini As IniFileType)

'***************************************************************
'* loads contents of .ini file into ini UDT structure.         *
'***************************************************************

   Dim IniNdx As Long, IniLine As String, FileNum As Integer

   On Error GoTo LoadError
'  null any data already stored in .ini udt (except filename).
   ClearIni Ini

   FileNum = FreeFile

'  if the file does not exist, create it and exit.
   If Not FileExists(Ini.FileName) Then
      Open Ini.FileName For Output As #FileNum
      Close #FileNum
      Exit Sub
   End If

   Open Ini.FileName For Input As #FileNum
'  read the .ini file and set up ini udt.
   IniNdx = 0
   Do While Not EOF(FileNum)
      IniNdx = IniNdx + 1
      ReDim Preserve Ini.IniFile(1 To IniNdx)
'     read in the line and place in udt's ini line array.
      Line Input #FileNum, IniLine
      Ini.IniFile(IniNdx) = Trim(IniLine)
'     if line is a section header, update the appropriate variables.
      If IsSectionHeader(Ini.IniFile(IniNdx)) Then
         Ini.SectionCount = Ini.SectionCount + 1
         ReDim Preserve Ini.SectionIndex(1 To Ini.SectionCount)
         ReDim Preserve Ini.SectionName(1 To Ini.SectionCount)
         ReDim Preserve Ini.NumItems(1 To Ini.SectionCount)
         Ini.SectionName(Ini.SectionCount) = Ini.IniFile(IniNdx)
         Ini.SectionIndex(Ini.SectionCount) = IniNdx
      Else
'        could be an item key in the current section, or a blank line or comment
'        (a line starting with ';') before the first section.
         If Ini.SectionCount > 0 Then
'           assumed to be an item key in the current section.  blank lines/comments between sections are
'           treated as an item key in the current section and are accounted for in ini manipulation code.
            Ini.NumItems(Ini.SectionCount) = Ini.NumItems(Ini.SectionCount) + 1
         End If
      End If
   Loop
   Close #FileNum

   Ini.TotalIniLines = IniNdx
   Exit Sub

LoadError:
   ErrorMessage = "Error " & CStr(Err.Number) & " (" & Err.Description & ") occurred while reading " & Ini.FileName & " from disk."
   DisplayErrorMessage ErrorMessage, "LoadIniFile"
   Ini.TotalIniLines = -1

End Sub

Friend Sub WriteIniFile(Ini As IniFileType)

'***************************************************************
'* store the ini structure to the actual .ini file.            *
'***************************************************************

   Dim IniLine As Long, FileNum As Integer

   On Error GoTo WriteToDiskError
   FileNum = FreeFile

   Open Ini.FileName For Output As #FileNum
   For IniLine = 1 To Ini.TotalIniLines
      Print #FileNum, Trim(Ini.IniFile(IniLine))
   Next IniLine
   Close #FileNum

   Exit Sub

WriteToDiskError:
   ErrorMessage = "Error " & CStr(Err.Number) & " (" & Err.Description & ") occurred while writing " & Ini.FileName & " to disk."
   DisplayErrorMessage ErrorMessage, "WriteIniFile"

End Sub

Friend Sub ClearIni(Ini As IniFileType)

'***************************************************************
'* nulls all aspects of ini user defined type except filename. *
'***************************************************************

   Erase Ini.IniFile
   ReDim Ini.IniFile(1 To 1)
   Erase Ini.SectionIndex
   ReDim Ini.SectionIndex(1 To 1)
   Erase Ini.NumItems
   ReDim Ini.NumItems(1 To 1)
   Erase Ini.SectionName
   ReDim Ini.SectionName(1 To 1)
   Ini.TotalIniLines = 0
   Ini.SectionCount = 0

End Sub

Friend Sub CreateNewSection(Ini As IniFileType, Section As String)

'***************************************************************
'* adds a new section header to the end of the ini UDT, adding *
'* a blank line first if necessary.                            *
'***************************************************************

   If Ini.TotalIniLines > 0 Then
'     add blank line between last item of last category and section header of new section.
      If Ini.IniFile(Ini.TotalIniLines) <> "" Then
         Ini.TotalIniLines = Ini.TotalIniLines + 1
         ReDim Preserve Ini.IniFile(1 To Ini.TotalIniLines)
         Ini.IniFile(Ini.TotalIniLines) = ""
      End If
   End If

'  add brackets to section header string if necessary.
   Bracketize Section
'  write the new section header line and update relevant data structures.
   Ini.TotalIniLines = Ini.TotalIniLines + 1
   ReDim Preserve Ini.IniFile(1 To Ini.TotalIniLines)
   Ini.IniFile(Ini.TotalIniLines) = Section
   Ini.SectionCount = Ini.SectionCount + 1
   ReDim Preserve Ini.NumItems(1 To Ini.SectionCount)
   ReDim Preserve Ini.SectionIndex(1 To Ini.SectionCount)
   ReDim Preserve Ini.SectionName(1 To Ini.SectionCount)
   Ini.NumItems(Ini.SectionCount) = 0
   Ini.SectionName(Ini.SectionCount) = Section
   Ini.SectionIndex(Ini.SectionCount) = Ini.TotalIniLines

End Sub

Friend Sub DeleteSection(Ini As IniFileType, Section As String)

'***************************************************************
'* deletes a section from ini structure.                       *
'***************************************************************

   Dim SectionNdx As Long, NumItemsInSectionToDelete As Long, NextSectionStartPos As Long
   Dim i As Long, SectionToDeleteStartPos As Long, SectionEndPos As Long

'  find the index of the appropriate section in the section index array.
   Bracketize Section
   SectionNdx = SectionExists(Ini, Section)
'  if the section doesn't exist, just exit.
   If SectionNdx = (-1) Then
      Exit Sub
   End If

'  if just one section header, just null all data in udt and exit.
   If Ini.SectionCount = 1 Then
      ClearIni Ini
      Exit Sub
   End If

'  if the section header is the last section header in the ini, make the line before the
'  section header the last line in the file, adjust other data items accordingly and exit.
   If SectionNdx = Ini.SectionCount Then
      Ini.TotalIniLines = Ini.SectionIndex(SectionNdx) - 1
      ReDim Preserve Ini.IniFile(1 To Ini.TotalIniLines)
      Ini.SectionCount = Ini.SectionCount - 1
      ReDim Preserve Ini.NumItems(1 To Ini.SectionCount)
      ReDim Preserve Ini.SectionIndex(1 To Ini.SectionCount)
      ReDim Preserve Ini.SectionName(1 To Ini.SectionCount)
      Exit Sub
   End If

'  if it's not the only section and not the last, get physical line number of section
'  header of the section immediately following section to delete.  move all lines from that
'  line to the end of the file to the positions starting with the line# of the section header to delete.
   SectionToDeleteStartPos = Ini.SectionIndex(SectionNdx)
   NumItemsInSectionToDelete = Ini.NumItems(SectionNdx)
   NextSectionStartPos = Ini.SectionIndex(SectionNdx + 1)
   For i = NextSectionStartPos To Ini.TotalIniLines
      Ini.IniFile(SectionToDeleteStartPos) = Ini.IniFile(i)
      SectionToDeleteStartPos = SectionToDeleteStartPos + 1
   Next i

'  adjust the total number of lines in the ini.  do this by subtracting # items in
'  the deleted section + 1 from # total lines.  '+ 1' takes section header into account.
   Ini.TotalIniLines = Ini.TotalIniLines - (Ini.NumItems(SectionNdx) + 1)
   ReDim Preserve Ini.IniFile(1 To Ini.TotalIniLines)
'  update the appropriate data structures.
   For i = SectionNdx To Ini.SectionCount - 1
      Ini.NumItems(i) = Ini.NumItems(i + 1)
      Ini.SectionIndex(i) = Ini.SectionIndex(i + 1)
'     set the new starting point of each section following the deleted one.
      Ini.SectionIndex(i) = Ini.SectionIndex(i) - NumItemsInSectionToDelete - 1
      Ini.SectionName(i) = Ini.SectionName(i + 1)
   Next i
   Ini.SectionCount = Ini.SectionCount - 1
   ReDim Preserve Ini.NumItems(1 To Ini.SectionCount)
   ReDim Preserve Ini.SectionIndex(1 To Ini.SectionCount)
   ReDim Preserve Ini.SectionName(1 To Ini.SectionCount)

End Sub

Friend Sub DeleteItem(Ini As IniFileType, Section As String, ItemKey As String)

'***************************************************************
'* deletes an item from specified section / itemkey.           *
'***************************************************************

   Dim SectionNdx As Long, ItemNdx As Long, i As Long

'  get index in SectionIndex array of desired section.  if section doesn't exist, exit.
   Bracketize Section
   SectionNdx = SectionExists(Ini, Section)
   If SectionNdx = (-1) Then
      Exit Sub
   End If

'  get the physical line# of item.  if the item doesn't exist, exit.
   ItemNdx = ItemExists(Ini, SectionNdx, ItemKey)
   If ItemNdx = (-1) Then
      Exit Sub
   End If

'  if item is last line in ini, it's very easy.
   If ItemNdx = Ini.TotalIniLines Then
      Ini.NumItems(SectionNdx) = Ini.NumItems(SectionNdx) - 1
      Ini.TotalIniLines = Ini.TotalIniLines - 1
      ReDim Preserve Ini.IniFile(1 To Ini.TotalIniLines)
      Exit Sub
   End If

'  otherwise, move all lines after item to delete up one slot.
   For i = ItemNdx To Ini.TotalIniLines - 1
      Ini.IniFile(i) = Ini.IniFile(i + 1)
   Next i
   Ini.TotalIniLines = Ini.TotalIniLines - 1
   ReDim Preserve Ini.IniFile(1 To Ini.TotalIniLines)
   Ini.NumItems(SectionNdx) = Ini.NumItems(SectionNdx) - 1

'  if the deleted item is in the last section, exit.
   If SectionNdx = Ini.SectionCount Then
      Exit Sub
   End If

'  otherwise, adjust line# of section headers following the section containing the deleted item.
   For i = SectionNdx + 1 To Ini.SectionCount
      Ini.SectionIndex(i) = Ini.SectionIndex(i) - 1
   Next i

End Sub

Private Function IsSectionHeader(IniLine As String) As Boolean

'***************************************************************
'* if it's surrounded by brackets, it's a section header.      *
'***************************************************************

   IsSectionHeader = (Left(IniLine, 1) = "[" And Right(IniLine, 1) = "]")

End Function

Private Sub Bracketize(ByRef Section As String)

'***************************************************************
'* adds brackets to supplied section name if they're absent.   *
'***************************************************************

   If Left(Section, 1) <> "[" Then Section = "[" & Trim(Section)
   If Right(Section, 1) <> "]" Then Section = Trim(Section) & "]"

End Sub

Private Function SectionExists(Ini As IniFileType, Section As String) As Long

'***************************************************************
'* determines if specified section header exists in .ini.      *
'* If section header is found, function returns the index of   *
'* the section header in the UDT's SectionIndex array.  If     *
'* there is no match, SectionExists returns -1.                *
'***************************************************************

   Dim i As Long, TempSectionName As String

   TempSectionName = UCase(Section) ' already arrives bracketized.
   For i = 1 To Ini.SectionCount
      If TempSectionName = UCase(Ini.SectionName(i)) Then
         SectionExists = i
         Exit Function
      End If
   Next i

'  if we get here, the section does not exist.
   SectionExists = -1

End Function

Private Function ItemExists(Ini As IniFileType, SectionNdx As Long, ItemKey As String) As Long

'***************************************************************
'* determines if the specified item key exists in the          *
'* .ini UDT.  If the item key is found, the function returns   *
'* the physical ini file line number location of the Item      *
'* header.  If there is no match, ItemExists returns -1.       *
'***************************************************************

   Dim i As Long, FirstItemInSection As Long, LastItemInSection As Long
   Dim SeparatorIndex As Long, TempItemKey As String

   TempItemKey = UCase(ItemKey)
   FirstItemInSection = Ini.SectionIndex(SectionNdx) + 1
   LastItemInSection = Ini.SectionIndex(SectionNdx) + Ini.NumItems(SectionNdx)

   For i = FirstItemInSection To LastItemInSection
      SeparatorIndex = InStr(Ini.IniFile(i), "=")
'     if the '=' is there, it's a valid item line.  otherwise, ignore the line.
      If SeparatorIndex > 0 Then
'        is the item we passed this function a match with the item key?
         If TempItemKey = UCase(Left(Ini.IniFile(i), SeparatorIndex - 1)) Then
'           if it is, return the inifile line number of the key and exit.
            ItemExists = i
            Exit Function
         End If
      End If
   Next i

'  if we get here, the item does not exist or there are no valid keys.
   ItemExists = -1

End Function

Private Function FileExists(PathFileName As String) As Boolean

'***************************************************************
'* simple file exists function.  Input is full path/filename.  *
'* found on PSC.  By Dan Redding, Blue Knot Software.          *
'***************************************************************

   On Error Resume Next
   FileExists = Dir(PathFileName, vbHidden) <> ""

End Function

Private Sub DisplayErrorMessage(ErrorMessage As String, ProcedureName As String)

'***************************************************************
'* displays error message generated by various routines.       *
'***************************************************************

   Dim i As Integer
   i = MsgBox(ErrorMessage, vbOKOnly + vbCritical, ".Ini Error in Procedure '" & ProcedureName & "'")

End Sub
